package domain;

import java.util.*;

import junit.framework.*;

public class CardTest extends TestCase {
   private Card card1;
   private Card card1copy;
   private Card card1copy2;
   private Card card2;
   private Card card3;
   
   protected void setUp() {
      card1 = new Card(Rank.king, Suit.hearts);
      card1copy = new Card(Rank.king, Suit.hearts);
      card1copy2 = new Card(Rank.king, Suit.hearts);
      card2 = new Card(Rank.queen, Suit.hearts);
      card3 = new Card(Rank.king, Suit.diamonds);
   }

   public void testCreate() {
      Card card = new Card(Rank.ace, Suit.clubs);
      assertEquals(Rank.ace, card.getRank());
      assertEquals(Suit.clubs, card.getSuit());
   }

   public void testEquality() {
      assertTrue(card1.equals(card1copy));
      assertFalse(card1.equals(card2));
      assertFalse(card1.equals(card3));
      Card card1subtype = new Card(Rank.king, Suit.hearts) {};
      assertFalse(card1.equals(card1subtype));

      assertReflexivity();
      assertSymmetry();
      assertTransitivity();
      assertConsistency();
      assertNullComparison();
   }
   
   public void testHashCode() {
      assertHashCodeConsistency();
      assertHashCodeEquality();
      assertHashCodePerformance();
   }
   
   private void assertHashCodePerformance() {
      final int capacity = 100000;
      final int iterations = capacity * 8 / 10 / 52;
      Set<Card> set = new HashSet<Card>(capacity);

      long start = System.currentTimeMillis();
      for (int i = 0; i < iterations; i++)
         for (Suit suit: Suit.values())
            for (Rank rank: Rank.values())
               set.add(new Card(rank, suit));
      long stop = System.currentTimeMillis();
      long elapsed = stop - start;
      assertTrue("elapsed: " + elapsed, elapsed < 50);
   }

   private void assertHashCodeEquality() {
      assertEquals(card1.hashCode(), card1copy.hashCode());
   }

   private void assertHashCodeConsistency() {
      assertEquals(card1.hashCode(), card1.hashCode());
   }

   private void assertNullComparison() {
      assertFalse(card1.equals(null));
   }

   private void assertConsistency() {
      assertTrue(card1.equals(card1copy));
      assertFalse(card1.equals(card2));
   }

   private void assertTransitivity() {
      assertTrue(card1copy.equals(card1copy2));
      assertTrue(card1.equals(card1copy2));
   }

   private void assertSymmetry() {
      assertTrue(card1.equals(card1copy));
      assertTrue(card1copy.equals(card1));
   }

   private void assertReflexivity() {
      assertTrue(card1.equals(card1));
   }
}